package com.qmth.boot.core.metrics.service.impl;

import com.qmth.boot.core.metrics.config.MetricsProperties;
import com.qmth.boot.core.metrics.model.MetricsResult;
import com.qmth.boot.core.metrics.model.TimeRange;

import java.time.Duration;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicLong;

public class MetricsRecorder {

    private Map<Integer, AtomicLong> statusMap;

    private List<TimeCounter> timeList;

    public MetricsRecorder(MetricsProperties properties) {
        this.statusMap = new HashMap<>();
        this.timeList = new ArrayList<>();
        long ge = 0;
        for (Duration time : properties.getTimeGroup()) {
            long lt = time.toMillis();
            this.timeList.add(new TimeCounter(ge, lt));
            ge = lt;
        }
        this.timeList.add(new TimeCounter(ge, -1));
    }

    public void record(int status, long timecost) {
        statusMap.computeIfAbsent(status, key -> new AtomicLong(0)).incrementAndGet();
        for (TimeCounter counter : timeList) {
            if (counter.accept(timecost)) {
                break;
            }
        }
    }

    public MetricsResult getResult() {
        MetricsResult result = new MetricsResult();

        Map<Integer, Long> statusResult = new HashMap<>();
        for (Map.Entry<Integer, AtomicLong> entry : statusMap.entrySet()) {
            statusResult.put(entry.getKey(), entry.getValue().longValue());
        }
        result.setStatus(statusResult);

        Map<TimeRange, Long> timeResult = new HashMap<>();
        for (TimeCounter tc : timeList) {
            timeResult.put(tc.getTimeRage(), tc.count());
        }
        result.setTime(timeResult);

        return result;
    }

}

